@@ -4,11 +4,10 @@
 # It also implements keyboard traversal of menus and implements a few
 # other utility procedures related to menus.
 #
-# RCS: @(#) $Id: menu.tcl,v 1.18 2002/08/31 06:12:28 das Exp $
-#
 # Copyright (c) 1992-1994 The Regents of the University of California.
 # Copyright (c) 1994-1997 Sun Microsystems, Inc.
 # Copyright (c) 1998-1999 by Scriptics Corporation.
+# Copyright (c) 2007 Daniel A. Steffen <das@users.sourceforge.net>
 #
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
@@ -61,7 +60,7 @@
 # This file is tricky because there are five different ways that menus
 # can be used:
 #
-# 1. As a pulldown from a menubutton. In this style, the variable 
+# 1. As a pulldown from a menubutton. In this style, the variable
 #    tk::Priv(postedMb) identifies the posted menubutton.
 # 2. As a torn-off menu copied from some other menu.  In this style
 #    tk::Priv(postedMb) is empty, and menu's type is "tearoff".
@@ -107,6 +106,10 @@ bind Menubutton <space> {
     tk::MbPost %W
     tk::MenuFirstEntry [%W cget -menu]
 }
+bind Menubutton <<Invoke>> {
+    tk::MbPost %W
+    tk::MenuFirstEntry [%W cget -menu]
+}
 
 # Must set focus when mouse enters a menu, in order to allow
 # mixed-mode processing using both the mouse and the keyboard.
@@ -144,22 +147,25 @@ bind Menu <ButtonRelease> {
 bind Menu <space> {
     tk::MenuInvoke %W 0
 }
+bind Menu <<Invoke>> {
+    tk::MenuInvoke %W 0
+}
 bind Menu <Return> {
     tk::MenuInvoke %W 0
 }
 bind Menu <Escape> {
     tk::MenuEscape %W
 }
-bind Menu <Left> {
+bind Menu <<PrevChar>> {
     tk::MenuLeftArrow %W
 }
-bind Menu <Right> {
+bind Menu <<NextChar>> {
     tk::MenuRightArrow %W
 }
-bind Menu <Up> {
+bind Menu <<PrevLine>> {
     tk::MenuUpArrow %W
 }
-bind Menu <Down> {
+bind Menu <<NextLine>> {
     tk::MenuDownArrow %W
 }
 bind Menu <KeyPress> {
@@ -169,7 +175,7 @@ bind Menu <KeyPress> {
 # The following bindings apply to all windows, and are used to
 # implement keyboard menu traversal.
 
-if {[string equal [tk windowingsystem] "x11"]} {
+if {[tk windowingsystem] eq "x11"} {
     bind all <Alt-KeyPress> {
 	tk::TraverseToMenu %W %A
     }
@@ -199,11 +205,11 @@ if {[string equal [tk windowingsystem] "x11"]} {
 proc ::tk::MbEnter w {
     variable ::tk::Priv
 
-    if {[string compare $Priv(inMenubutton) ""]} {
+    if {$Priv(inMenubutton) ne ""} {
 	MbLeave $Priv(inMenubutton)
     }
     set Priv(inMenubutton) $w
-    if {[string compare [$w cget -state] "disabled"]} {
+    if {[$w cget -state] ne "disabled" && [tk windowingsystem] ne "aqua"} {
 	$w configure -state active
     }
 }
@@ -222,7 +228,7 @@ proc ::tk::MbLeave w {
     if {![winfo exists $w]} {
 	return
     }
-    if {[string equal [$w cget -state] "active"]} {
+    if {[$w cget -state] eq "active" && [tk windowingsystem] ne "aqua"} {
 	$w configure -state normal
     }
 }
@@ -248,22 +254,29 @@ proc ::tk::MbPost {w {x {}} {y {}}} {
 	return
     }
     set menu [$w cget -menu]
-    if {[string equal $menu ""]} {
+    if {$menu eq ""} {
 	return
     }
     set tearoff [expr {[tk windowingsystem] eq "x11" \
 	    || [$menu cget -type] eq "tearoff"}]
     if {[string first $w $menu] != 0} {
-	error "can't post $menu:  it isn't a descendant of $w (this is a new requirement in Tk versions 3.0 and later)"
+	return -code error -errorcode {TK MENUBUTTON POST_NONCHILD} \
+	    "can't post $menu: it isn't a descendant of $w"
     }
     set cur $Priv(postedMb)
-    if {[string compare $cur ""]} {
+    if {$cur ne ""} {
 	MenuUnpost {}
     }
-    set Priv(cursor) [$w cget -cursor]
-    set Priv(relief) [$w cget -relief]
-    $w configure -cursor arrow
-    $w configure -relief raised
+    if {$::tk_strictMotif} {
+        set Priv(cursor) [$w cget -cursor]
+        $w configure -cursor arrow
+    }
+    if {[tk windowingsystem] ne "aqua"} {
+	set Priv(relief) [$w cget -relief]
+	$w configure -relief raised
+    } else {
+	$w configure -state active
+    }
 
     set Priv(postedMb) $w
     set Priv(focus) [focus]
@@ -277,81 +290,88 @@ proc ::tk::MbPost {w {x {}} {y {}}} {
     update idletasks
     if {[catch {
 	switch [$w cget -direction] {
-    	    above {
-    	    	set x [winfo rootx $w]
-    	    	set y [expr {[winfo rooty $w] - [winfo reqheight $menu]}]
+	    above {
+		set x [winfo rootx $w]
+		set y [expr {[winfo rooty $w] - [winfo reqheight $menu]}]
+		# if we go offscreen to the top, show as 'below'
+		if {$y < [winfo vrooty $w]} {
+		    set y [expr {[winfo vrooty $w] + [winfo rooty $w] + [winfo reqheight $w]}]
+		}
 		PostOverPoint $menu $x $y
-    	    }
-    	    below {
-    	    	set x [winfo rootx $w]
-    	    	set y [expr {[winfo rooty $w] + [winfo height $w]}]
+	    }
+	    below {
+		set x [winfo rootx $w]
+		set y [expr {[winfo rooty $w] + [winfo height $w]}]
+		# if we go offscreen to the bottom, show as 'above'
+		set mh [winfo reqheight $menu]
+		if {($y + $mh) > ([winfo vrooty $w] + [winfo vrootheight $w])} {
+		    set y [expr {[winfo vrooty $w] + [winfo vrootheight $w] + [winfo rooty $w] - $mh}]
+		}
 		PostOverPoint $menu $x $y
-    	    }
-    	    left {
-    	    	set x [expr {[winfo rootx $w] - [winfo reqwidth $menu]}]
-    	    	set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
-    	    	set entry [MenuFindName $menu [$w cget -text]]
-    	    	if {[$w cget -indicatoron]} {
+	    }
+	    left {
+		set x [expr {[winfo rootx $w] - [winfo reqwidth $menu]}]
+		set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
+		set entry [MenuFindName $menu [$w cget -text]]
+		if {[$w cget -indicatoron]} {
 		    if {$entry == [$menu index last]} {
-		    	incr y [expr {-([$menu yposition $entry] \
-			    	+ [winfo reqheight $menu])/2}]
+			incr y [expr {-([$menu yposition $entry] \
+				+ [winfo reqheight $menu])/2}]
 		    } else {
-		    	incr y [expr {-([$menu yposition $entry] \
+			incr y [expr {-([$menu yposition $entry] \
 			        + [$menu yposition [expr {$entry+1}]])/2}]
 		    }
-    	    	}
+		}
 		PostOverPoint $menu $x $y
 		if {$entry ne "" \
 			&& [$menu entrycget $entry -state] ne "disabled"} {
-    	    	    $menu activate $entry
+		    $menu activate $entry
 		    GenerateMenuSelect $menu
-    	    	}
-    	    }
-    	    right {
-    	    	set x [expr {[winfo rootx $w] + [winfo width $w]}]
-    	    	set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
-    	    	set entry [MenuFindName $menu [$w cget -text]]
-    	    	if {[$w cget -indicatoron]} {
+		}
+	    }
+	    right {
+		set x [expr {[winfo rootx $w] + [winfo width $w]}]
+		set y [expr {(2 * [winfo rooty $w] + [winfo height $w]) / 2}]
+		set entry [MenuFindName $menu [$w cget -text]]
+		if {[$w cget -indicatoron]} {
 		    if {$entry == [$menu index last]} {
-		    	incr y [expr {-([$menu yposition $entry] \
-			    	+ [winfo reqheight $menu])/2}]
+			incr y [expr {-([$menu yposition $entry] \
+				+ [winfo reqheight $menu])/2}]
 		    } else {
-		    	incr y [expr {-([$menu yposition $entry] \
+			incr y [expr {-([$menu yposition $entry] \
 			        + [$menu yposition [expr {$entry+1}]])/2}]
 		    }
-    	    	}
+		}
 		PostOverPoint $menu $x $y
 		if {$entry ne "" \
 			&& [$menu entrycget $entry -state] ne "disabled"} {
-    	    	    $menu activate $entry
+		    $menu activate $entry
 		    GenerateMenuSelect $menu
-    	    	}
-    	    }
-    	    default {
-    	    	if {[$w cget -indicatoron]} {
-		    if {[string equal $y {}]} {
+		}
+	    }
+	    default {
+		if {[$w cget -indicatoron]} {
+		    if {$y eq ""} {
 			set x [expr {[winfo rootx $w] + [winfo width $w]/2}]
 			set y [expr {[winfo rooty $w] + [winfo height $w]/2}]
-	    	    }
+		    }
 	            PostOverPoint $menu $x $y [MenuFindName $menu [$w cget -text]]
 		} else {
 		    PostOverPoint $menu [winfo rootx $w] [expr {[winfo rooty $w]+[winfo height $w]}]
-    	    	}  
-    	    }
+		}
+	    }
 	}
-    } msg]} {
+    } msg opt]} {
 	# Error posting menu (e.g. bogus -postcommand). Unpost it and
 	# reflect the error.
-	
-	set savedInfo $errorInfo
-	MenuUnpost {}
-	error $msg $savedInfo
 
+	MenuUnpost {}
+	return -options $opt $msg
     }
 
     set Priv(tearoff) $tearoff
     if {$tearoff != 0} {
-    	focus $menu
+	focus $menu
 	if {[winfo viewable $w]} {
 	    SaveGrabInfo $w
 	    grab -global $w
@@ -390,18 +410,28 @@ proc ::tk::MenuUnpost menu {
     # Unpost menu(s) and restore some stuff that's dependent on
     # what was posted.
 
+    after cancel [array get Priv menuActivatedTimer]
+    unset -nocomplain Priv(menuActivated)
+    after cancel [array get Priv menuDeactivatedTimer]
+    unset -nocomplain Priv(menuDeactivated)
+
     catch {
-	if {[string compare $mb ""]} {
+	if {$mb ne ""} {
 	    set menu [$mb cget -menu]
 	    $menu unpost
 	    set Priv(postedMb) {}
-	    $mb configure -cursor $Priv(cursor)
-	    $mb configure -relief $Priv(relief)
-	} elseif {[string compare $Priv(popup) ""]} {
+	    if {$::tk_strictMotif} {
+	        $mb configure -cursor $Priv(cursor)
+	    }
+	    if {[tk windowingsystem] ne "aqua"} {
+		$mb configure -relief $Priv(relief)
+	    } else {
+		$mb configure -state normal
+	    }
+	} elseif {$Priv(popup) ne ""} {
 	    $Priv(popup) unpost
 	    set Priv(popup) {}
-	} elseif {[string compare [$menu cget -type] "menubar"] \
-		&& [string compare [$menu cget -type] "tearoff"]} {
+	} elseif {[$menu cget -type] ne "menubar" && [$menu cget -type] ne "tearoff"} {
 	    # We're in a cascaded sub-menu from a torn-off menu or popup.
 	    # Unpost all the menus up to the toplevel one (but not
 	    # including the top-level torn-off one) and deactivate the
@@ -409,38 +439,38 @@ proc ::tk::MenuUnpost menu {
 
 	    while {1} {
 		set parent [winfo parent $menu]
-		if {[string compare [winfo class $parent] "Menu"] \
-			|| ![winfo ismapped $parent]} {
+		if {[winfo class $parent] ne "Menu" || ![winfo ismapped $parent]} {
 		    break
 		}
 		$parent activate none
 		$parent postcascade none
 		GenerateMenuSelect $parent
 		set type [$parent cget -type]
-		if {[string equal $type "menubar"] || \
-			[string equal $type "tearoff"]} {
+		if {$type eq "menubar" || $type eq "tearoff"} {
 		    break
 		}
 		set menu $parent
 	    }
-	    if {[string compare [$menu cget -type] "menubar"]} {
+	    if {[$menu cget -type] ne "menubar"} {
 		$menu unpost
 	    }
 	}
     }
 
     if {($Priv(tearoff) != 0) || $Priv(menuBar) ne ""} {
-    	# Release grab, if any, and restore the previous grab, if there
-    	# was one.
-	if {[string compare $menu ""]} {
+	# Release grab, if any, and restore the previous grab, if there
+	# was one.
+	if {$menu ne ""} {
 	    set grab [grab current $menu]
-	    if {[string compare $grab ""]} {
+	    if {$grab ne ""} {
 		grab release $grab
 	    }
 	}
 	RestoreOldGrab
 	if {$Priv(menuBar) ne ""} {
-	    $Priv(menuBar) configure -cursor $Priv(cursor)
+	    if {$::tk_strictMotif} {
+		$Priv(menuBar) configure -cursor $Priv(cursor)
+	    }
 	    set Priv(menuBar) {}
 	}
 	if {[tk windowingsystem] ne "x11"} {
@@ -463,21 +493,20 @@ proc ::tk::MenuUnpost menu {
 proc ::tk::MbMotion {w upDown rootx rooty} {
     variable ::tk::Priv
 
-    if {[string equal $Priv(inMenubutton) $w]} {
+    if {$Priv(inMenubutton) eq $w} {
 	return
     }
     set new [winfo containing $rootx $rooty]
-    if {[string compare $new $Priv(inMenubutton)] \
-	    && ([string equal $new ""] \
-	    || [string equal [winfo toplevel $new] [winfo toplevel $w]])} {
-	if {[string compare $Priv(inMenubutton) ""]} {
+    if {$new ne $Priv(inMenubutton) \
+	    && ($new eq "" || [winfo toplevel $new] eq [winfo toplevel $w])} {
+	if {$Priv(inMenubutton) ne ""} {
 	    MbLeave $Priv(inMenubutton)
 	}
-	if {[string compare $new ""] \
-		&& [string equal [winfo class $new] "Menubutton"] \
+	if {$new ne "" \
+		&& [winfo class $new] eq "Menubutton" \
 		&& ([$new cget -indicatoron] == 0) \
 		&& ([$w cget -indicatoron] == 0)} {
-	    if {[string equal $upDown "down"]} {
+	    if {$upDown eq "down"} {
 		MbPost $new $rootx $rooty
 	    } else {
 		MbEnter $new
@@ -524,10 +553,10 @@ proc ::tk::MbButtonUp w {
 
 proc ::tk::MenuMotion {menu x y state} {
     variable ::tk::Priv
-    if {[string equal $menu $Priv(window)]} {
-	if {[string equal [$menu cget -type] "menubar"]} {
-	    if {[info exists Priv(focus)] && \
-		    [string compare $menu $Priv(focus)]} {
+    if {$menu eq $Priv(window)} {
+        set activeindex [$menu index active]
+	if {[$menu cget -type] eq "menubar"} {
+	    if {[info exists Priv(focus)] && $menu ne $Priv(focus)} {
 		$menu activate @$x,$y
 		GenerateMenuSelect $menu
 	    }
@@ -535,9 +564,22 @@ proc ::tk::MenuMotion {menu x y state} {
 	    $menu activate @$x,$y
 	    GenerateMenuSelect $menu
 	}
-    }
-    if {($state & 0x1f00) != 0} {
-	$menu postcascade active
+        set index [$menu index @$x,$y]
+        if {[info exists Priv(menuActivated)] \
+                && $index ne "none" \
+                && $index ne $activeindex} {
+            set mode [option get $menu clickToFocus ClickToFocus]
+            if {[string is false $mode]} {
+                set delay [expr {[$menu cget -type] eq "menubar" ? 0 : 50}]
+                if {[$menu type $index] eq "cascade"} {
+                    set Priv(menuActivatedTimer) \
+                        [after $delay [list $menu postcascade active]]
+                } else {
+                    set Priv(menuDeactivatedTimer) \
+                        [after $delay [list $menu postcascade none]]
+                }
+            }
+        }
     }
 }
 
@@ -564,20 +606,24 @@ proc ::tk::MenuButtonDown menu {
         return
     }
     $menu postcascade active
-    if {[string compare $Priv(postedMb) ""] && \
-	    [winfo viewable $Priv(postedMb)]} {
+    if {$Priv(postedMb) ne "" && [winfo viewable $Priv(postedMb)]} {
 	grab -global $Priv(postedMb)
     } else {
-	while {[string equal [$menu cget -type] "normal"] \
-		&& [string equal [winfo class [winfo parent $menu]] "Menu"] \
+	while {[$menu cget -type] eq "normal" \
+		&& [winfo class [winfo parent $menu]] eq "Menu" \
 		&& [winfo ismapped [winfo parent $menu]]} {
 	    set menu [winfo parent $menu]
 	}
 
-	if {[string equal $Priv(menuBar) {}]} {
+	if {$Priv(menuBar) eq {}} {
 	    set Priv(menuBar) $menu
-	    set Priv(cursor) [$menu cget -cursor]
-	    $menu configure -cursor arrow
+	    if {$::tk_strictMotif} {
+		set Priv(cursor) [$menu cget -cursor]
+		$menu configure -cursor arrow
+	    }
+	    if {[$menu type active] eq "cascade"} {
+		set Priv(menuActivated) 1
+	    }
         }
 
 	# Don't update grab information if the grab window isn't changing.
@@ -585,14 +631,14 @@ proc ::tk::MenuButtonDown menu {
 	# restore the grab, since the old grab window will not be viewable
 	# anymore.
 
-	if {[string compare $menu [grab current $menu]]} {
+	if {$menu ne [grab current $menu]} {
 	    SaveGrabInfo $menu
 	}
 
 	# Must re-grab even if the grab window hasn't changed, in order
 	# to release the implicit grab from the button press.
 
-	if {[string equal [tk windowingsystem] "x11"]} {
+	if {[tk windowingsystem] eq "x11"} {
 	    grab -global $menu
 	}
     }
@@ -611,12 +657,12 @@ proc ::tk::MenuButtonDown menu {
 proc ::tk::MenuLeave {menu rootx rooty state} {
     variable ::tk::Priv
     set Priv(window) {}
-    if {[string equal [$menu index active] "none"]} {
+    if {[$menu index active] eq "none"} {
 	return
     }
-    if {[string equal [$menu type active] "cascade"]
-          && [string equal [winfo containing $rootx $rooty] \
-                  [$menu entrycget active -menu]]} {
+    if {[$menu type active] eq "cascade" \
+	    && [winfo containing $rootx $rooty] eq \
+		[$menu entrycget active -menu]} {
 	return
     }
     $menu activate none
@@ -636,7 +682,7 @@ proc ::tk::MenuLeave {menu rootx rooty state} {
 proc ::tk::MenuInvoke {w buttonRelease} {
     variable ::tk::Priv
 
-    if {$buttonRelease && [string equal $Priv(window) {}]} {
+    if {$buttonRelease && $Priv(window) eq ""} {
 	# Mouse was pressed over a menu without a menu button, then
 	# dragged off the menu (possibly with a cascade posted) and
 	# released.  Unpost everything and quit.
@@ -647,20 +693,20 @@ proc ::tk::MenuInvoke {w buttonRelease} {
 	MenuUnpost $w
 	return
     }
-    if {[string equal [$w type active] "cascade"]} {
+    if {[$w type active] eq "cascade"} {
 	$w postcascade active
 	set menu [$w entrycget active -menu]
 	MenuFirstEntry $menu
-    } elseif {[string equal [$w type active] "tearoff"]} {
+    } elseif {[$w type active] eq "tearoff"} {
 	::tk::TearOffMenu $w
 	MenuUnpost $w
-    } elseif {[string equal [$w cget -type] "menubar"]} {
+    } elseif {[$w cget -type] eq "menubar"} {
 	$w postcascade none
 	set active [$w index active]
 	set isCascade [string equal [$w type $active] "cascade"]
 
 	# Only de-activate the active item if it's a cascade; this prevents
-	# the annoying "activation flicker" you otherwise get with 
+	# the annoying "activation flicker" you otherwise get with
 	# checkbuttons/commands/etc. on menubars
 
 	if { $isCascade } {
@@ -678,7 +724,10 @@ proc ::tk::MenuInvoke {w buttonRelease} {
 	    uplevel #0 [list $w invoke $active]
 	}
     } else {
-	MenuUnpost $w
+	set active [$w index active]
+	if {$Priv(popup) eq "" || $active ne "none"} {
+	    MenuUnpost $w
+	}
 	uplevel #0 [list $w invoke active]
     }
 }
@@ -693,9 +742,9 @@ proc ::tk::MenuInvoke {w buttonRelease} {
 
 proc ::tk::MenuEscape menu {
     set parent [winfo parent $menu]
-    if {[string compare [winfo class $parent] "Menu"]} {
+    if {[winfo class $parent] ne "Menu"} {
 	MenuUnpost $menu
-    } elseif {[string equal [$parent cget -type] "menubar"]} {
+    } elseif {[$parent cget -type] eq "menubar"} {
 	MenuUnpost $menu
 	RestoreOldGrab
     } else {
@@ -707,7 +756,7 @@ proc ::tk::MenuEscape menu {
 # differently depending on whether the menu is a menu bar or not.
 
 proc ::tk::MenuUpArrow {menu} {
-    if {[string equal [$menu cget -type] "menubar"]} {
+    if {[$menu cget -type] eq "menubar"} {
 	MenuNextMenu $menu left
     } else {
 	MenuNextEntry $menu -1
@@ -715,7 +764,7 @@ proc ::tk::MenuUpArrow {menu} {
 }
 
 proc ::tk::MenuDownArrow {menu} {
-    if {[string equal [$menu cget -type] "menubar"]} {
+    if {[$menu cget -type] eq "menubar"} {
 	MenuNextMenu $menu right
     } else {
 	MenuNextEntry $menu 1
@@ -723,7 +772,7 @@ proc ::tk::MenuDownArrow {menu} {
 }
 
 proc ::tk::MenuLeftArrow {menu} {
-    if {[string equal [$menu cget -type] "menubar"]} {
+    if {[$menu cget -type] eq "menubar"} {
 	MenuNextEntry $menu -1
     } else {
 	MenuNextMenu $menu left
@@ -731,7 +780,7 @@ proc ::tk::MenuLeftArrow {menu} {
 }
 
 proc ::tk::MenuRightArrow {menu} {
-    if {[string equal [$menu cget -type] "menubar"]} {
+    if {[$menu cget -type] eq "menubar"} {
 	MenuNextEntry $menu 1
     } else {
 	MenuNextMenu $menu right
@@ -753,22 +802,22 @@ proc ::tk::MenuNextMenu {menu direction} {
 
     # First handle traversals into and out of cascaded menus.
 
-    if {[string equal $direction "right"]} {
+    if {$direction eq "right"} {
 	set count 1
 	set parent [winfo parent $menu]
 	set class [winfo class $parent]
-	if {[string equal [$menu type active] "cascade"]} {
+	if {[$menu type active] eq "cascade"} {
 	    $menu postcascade active
 	    set m2 [$menu entrycget active -menu]
-	    if {[string compare $m2 ""]} {
+	    if {$m2 ne ""} {
 		MenuFirstEntry $m2
 	    }
 	    return
 	} else {
 	    set parent [winfo parent $menu]
-	    while {[string compare $parent "."]} {
-		if {[string equal [winfo class $parent] "Menu"] \
-			&& [string equal [$parent cget -type] "menubar"]} {
+	    while {$parent ne "."} {
+		if {[winfo class $parent] eq "Menu" \
+			&& [$parent cget -type] eq "menubar"} {
 		    tk_menuSetFocus $parent
 		    MenuNextEntry $parent 1
 		    return
@@ -779,33 +828,31 @@ proc ::tk::MenuNextMenu {menu direction} {
     } else {
 	set count -1
 	set m2 [winfo parent $menu]
-	if {[string equal [winfo class $m2] "Menu"]} {
+	if {[winfo class $m2] eq "Menu"} {
 	    $menu activate none
 	    GenerateMenuSelect $menu
 	    tk_menuSetFocus $m2
 
 	    $m2 postcascade none
 
-	    if {[string compare [$m2 cget -type] "menubar"]} {
+	    if {[$m2 cget -type] ne "menubar"} {
 		return
 	    }
 	}
     }
 
-    # Can't traverse into or out of a cascaded menu.  Go to the next
+    # Can't traverse into or out of a cascaded menu. Go to the next
     # or previous menubutton, if that makes sense.
 
     set m2 [winfo parent $menu]
-    if {[string equal [winfo class $m2] "Menu"]} {
-	if {[string equal [$m2 cget -type] "menubar"]} {
-	    tk_menuSetFocus $m2
-	    MenuNextEntry $m2 -1
-	    return
-	}
+    if {[winfo class $m2] eq "Menu" && [$m2 cget -type] eq "menubar"} {
+	tk_menuSetFocus $m2
+	MenuNextEntry $m2 -1
+	return
     }
 
     set w $Priv(postedMb)
-    if {[string equal $w ""]} {
+    if {$w eq ""} {
 	return
     }
     set buttons [winfo children [winfo parent $w]]
@@ -819,13 +866,13 @@ proc ::tk::MenuNextMenu {menu direction} {
 	    incr i -$length
 	}
 	set mb [lindex $buttons $i]
-	if {[string equal [winfo class $mb] "Menubutton"] \
-		&& [string compare [$mb cget -state] "disabled"] \
-		&& [string compare [$mb cget -menu] ""] \
-		&& [string compare [[$mb cget -menu] index last] "none"]} {
+	if {[winfo class $mb] eq "Menubutton" \
+		&& [$mb cget -state] ne "disabled" \
+		&& [$mb cget -menu] ne "" \
+		&& [[$mb cget -menu] index last] ne "none"} {
 	    break
 	}
-	if {[string equal $mb $w]} {
+	if {$mb eq $w} {
 	    return
 	}
 	incr i $count
@@ -844,14 +891,13 @@ proc ::tk::MenuNextMenu {menu direction} {
 #				-1 means go to the next higher entry.
 
 proc ::tk::MenuNextEntry {menu count} {
-
-    if {[string equal [$menu index last] "none"]} {
+    if {[$menu index last] eq "none"} {
 	return
     }
     set length [expr {[$menu index last]+1}]
     set quitAfter $length
     set active [$menu index active]
-    if {[string equal $active "none"]} {
+    if {$active eq "none"} {
 	set i 0
     } else {
 	set i [expr {$active + $count}]
@@ -885,10 +931,9 @@ proc ::tk::MenuNextEntry {menu count} {
     $menu activate $i
     GenerateMenuSelect $menu
 
-    if {[string equal [$menu type $i] "cascade"] \
-	    && [string equal [$menu cget -type] "menubar"]} {
+    if {[$menu type $i] eq "cascade" && [$menu cget -type] eq "menubar"} {
 	set cascade [$menu entrycget $i -menu]
-	if {[string compare $cascade ""]} {
+	if {$cascade ne ""} {
 	    # Here we auto-post a cascade.  This is necessary when
 	    # we traverse left/right in the menubar, but undesirable when
 	    # we traverse up/down in a menu.
@@ -920,24 +965,23 @@ proc ::tk::MenuFind {w char} {
 
     foreach child $windowlist {
 	# Don't descend into other toplevels.
-        if {[string compare [winfo toplevel $w] [winfo toplevel $child]]} {
+        if {[winfo toplevel $w] ne [winfo toplevel $child]} {
 	    continue
 	}
-	if {[string equal [winfo class $child] "Menu"] && \
-		[string equal [$child cget -type] "menubar"]} {
-	    if {[string equal $char ""]} {
+	if {[winfo class $child] eq "Menu" && \
+		[$child cget -type] eq "menubar"} {
+	    if {$char eq ""} {
 		return $child
 	    }
 	    set last [$child index last]
 	    for {set i [$child cget -tearoff]} {$i <= $last} {incr i} {
-		if {[string equal [$child type $i] "separator"]} {
+		if {[$child type $i] eq "separator"} {
 		    continue
 		}
 		set char2 [string index [$child entrycget $i -label] \
 			[$child entrycget $i -underline]]
-		if {[string equal $char [string tolower $char2]] \
-			|| [string equal $char ""]} {
-		    if {[string compare [$child entrycget $i -state] "disabled"]} {
+		if {$char eq [string tolower $char2] || $char eq ""} {
+		    if {[$child entrycget $i -state] ne "disabled"} {
 			return $child
 		    }
 		}
@@ -947,16 +991,15 @@ proc ::tk::MenuFind {w char} {
 
     foreach child $windowlist {
 	# Don't descend into other toplevels.
-        if {[string compare [winfo toplevel $w] [winfo toplevel $child]]} {
+        if {[winfo toplevel $w] ne [winfo toplevel $child]} {
 	    continue
 	}
-	switch [winfo class $child] {
+	switch -- [winfo class $child] {
 	    Menubutton {
 		set char2 [string index [$child cget -text] \
 			[$child cget -underline]]
-		if {[string equal $char [string tolower $char2]] \
-			|| [string equal $char ""]} {
-		    if {[string compare [$child cget -state] "disabled"]} {
+		if {$char eq [string tolower $char2] || $char eq ""} {
+		    if {[$child cget -state] ne "disabled"} {
 			return $child
 		    }
 		}
@@ -964,7 +1007,7 @@ proc ::tk::MenuFind {w char} {
 
 	    default {
 		set match [MenuFind $child $char]
-		if {[string compare $match ""]} {
+		if {$match ne ""} {
 		    return $match
 		}
 	    }
@@ -987,22 +1030,20 @@ proc ::tk::MenuFind {w char} {
 
 proc ::tk::TraverseToMenu {w char} {
     variable ::tk::Priv
-    if {[string equal $char ""]} {
+    if {$char eq ""} {
 	return
     }
-    while {[string equal [winfo class $w] "Menu"]} {
-	if {[string compare [$w cget -type] "menubar"] \
-		&& [string equal $Priv(postedMb) ""]} {
-	    return
-	}
-	if {[string equal [$w cget -type] "menubar"]} {
+    while {[winfo class $w] eq "Menu"} {
+	if {[$w cget -type] eq "menubar"} {
 	    break
+	} elseif {$Priv(postedMb) eq ""} {
+	    return
 	}
 	set w [winfo parent $w]
     }
     set w [MenuFind [winfo toplevel $w] $char]
-    if {[string compare $w ""]} {
-	if {[string equal [winfo class $w] "Menu"]} {
+    if {$w ne ""} {
+	if {[winfo class $w] eq "Menu"} {
 	    tk_menuSetFocus $w
 	    set Priv(window) $w
 	    SaveGrabInfo $w
@@ -1026,8 +1067,8 @@ proc ::tk::TraverseToMenu {w char} {
 proc ::tk::FirstMenu w {
     variable ::tk::Priv
     set w [MenuFind [winfo toplevel $w] ""]
-    if {[string compare $w ""]} {
-	if {[string equal [winfo class $w] "Menu"]} {
+    if {$w ne ""} {
+	if {[winfo class $w] eq "Menu"} {
 	    tk_menuSetFocus $w
 	    set Priv(window) $w
 	    SaveGrabInfo $w
@@ -1052,12 +1093,12 @@ proc ::tk::FirstMenu w {
 #				nothing happens.
 
 proc ::tk::TraverseWithinMenu {w char} {
-    if {[string equal $char ""]} {
+    if {$char eq ""} {
 	return
     }
     set char [string tolower $char]
     set last [$w index last]
-    if {[string equal $last "none"]} {
+    if {$last eq "none"} {
 	return
     }
     for {set i 0} {$i <= $last} {incr i} {
@@ -1065,13 +1106,13 @@ proc ::tk::TraverseWithinMenu {w char} {
 		[$w entrycget $i -label] [$w entrycget $i -underline]]}]} {
 	    continue
 	}
-	if {[string equal $char [string tolower $char2]]} {
-	    if {[string equal [$w type $i] "cascade"]} {
+	if {$char eq [string tolower $char2]} {
+	    if {[$w type $i] eq "cascade"} {
 		$w activate $i
 		$w postcascade active
 		event generate $w <<MenuSelect>>
 		set m2 [$w entrycget $i -menu]
-		if {[string compare $m2 ""]} {
+		if {$m2 ne ""} {
 		    MenuFirstEntry $m2
 		}
 	    } else {
@@ -1095,31 +1136,29 @@ proc ::tk::TraverseWithinMenu {w char} {
 # menu -		Name of the menu window (possibly empty).
 
 proc ::tk::MenuFirstEntry menu {
-    if {[string equal $menu ""]} {
+    if {$menu eq ""} {
 	return
     }
     tk_menuSetFocus $menu
-    if {[string compare [$menu index active] "none"]} {
+    if {[$menu index active] ne "none"} {
 	return
     }
     set last [$menu index last]
-    if {[string equal $last "none"]} {
+    if {$last eq "none"} {
 	return
     }
     for {set i 0} {$i <= $last} {incr i} {
 	if {([catch {set state [$menu entrycget $i -state]}] == 0) \
-		&& [string compare $state "disabled"] \
-		&& [string compare [$menu type $i] "tearoff"]} {
+		&& $state ne "disabled" && [$menu type $i] ne "tearoff"} {
 	    $menu activate $i
 	    GenerateMenuSelect $menu
 	    # Only post the cascade if the current menu is a menubar;
 	    # otherwise, if the first entry of the cascade is a cascade,
 	    # we can get an annoying cascading effect resulting in a bunch of
 	    # menus getting posted (bug 676)
-	    if {[string equal [$menu type $i] "cascade"] && \
-		[string equal [$menu cget -type] "menubar"]} {
+	    if {[$menu type $i] eq "cascade" && [$menu cget -type] eq "menubar"} {
 		set cascade [$menu entrycget $i -menu]
-		if {[string compare $cascade ""]} {
+		if {$cascade ne ""} {
 		    $menu postcascade $i
 		    MenuFirstEntry $cascade
 		}
@@ -1147,12 +1186,12 @@ proc ::tk::MenuFindName {menu s} {
 	return $i
     }
     set last [$menu index last]
-    if {[string equal $last "none"]} {
+    if {$last eq "none"} {
 	return
     }
     for {set i 0} {$i <= $last} {incr i} {
 	if {![catch {$menu entrycget $i -label} label]} {
-	    if {[string equal $label $s]} {
+	    if {$label eq $s} {
 		return $i
 	    }
 	}
@@ -1174,8 +1213,8 @@ proc ::tk::MenuFindName {menu s} {
 
 proc ::tk::PostOverPoint {menu x y {entry {}}}  {
     global tcl_platform
-    
-    if {[string compare $entry {}]} {
+
+    if {$entry ne ""} {
 	if {$entry == [$menu index last]} {
 	    incr y [expr {-([$menu yposition $entry] \
 		    + [winfo reqheight $menu])/2}]
@@ -1185,20 +1224,34 @@ proc ::tk::PostOverPoint {menu x y {entry {}}}  {
 	}
 	incr x [expr {-[winfo reqwidth $menu]/2}]
     }
-    if {$tcl_platform(platform) == "windows"} {
-	# We need to fix some problems with menu posting on Windows.
-	set yoffset [expr {[winfo screenheight $menu] \
-		- $y - [winfo reqheight $menu]}]
-	if {$yoffset < 0} {
-	    # The bottom of the menu is offscreen, so adjust upwards
-	    incr y $yoffset
-	    if {$y < 0} { set y 0 }
+
+    if {[tk windowingsystem] eq "win32"} {
+	# osVersion is not available in safe interps
+	set ver 5
+	if {[info exists tcl_platform(osVersion)]} {
+	    scan $tcl_platform(osVersion) %d ver
 	}
-	# If we're off the top of the screen (either because we were
-	# originally or because we just adjusted too far upwards),
-	# then make the menu popup on the top edge.
-	if {$y < 0} {
-	    set y 0
+
+	# We need to fix some problems with menu posting on Windows,
+	# where, if the menu would overlap top or bottom of screen,
+	# Windows puts it in the wrong place for us.  We must also
+	# subtract an extra amount for half the height of the current
+	# entry.  To be safe we subtract an extra 10.
+	# NOTE: this issue appears to have been resolved in the Window
+	# manager provided with Vista and Windows 7.
+	if {$ver < 6} {
+	    set yoffset [expr {[winfo screenheight $menu] \
+		    - $y - [winfo reqheight $menu] - 10}]
+	    if {$yoffset < [winfo vrooty $menu]} {
+		# The bottom of the menu is offscreen, so adjust upwards
+		incr y [expr {$yoffset - [winfo vrooty $menu]}]
+	    }
+	    # If we're off the top of the screen (either because we were
+	    # originally or because we just adjusted too far upwards),
+	    # then make the menu popup on the top edge.
+	    if {$y < [winfo vrooty $menu]} {
+		set y [winfo vrooty $menu]
+	    }
 	}
     }
     $menu post $x $y
@@ -1232,11 +1285,11 @@ proc ::tk::RestoreOldGrab {} {
     variable ::tk::Priv
 
     if {$Priv(oldGrab) ne ""} {
-    	# Be careful restoring the old grab, since it's window may not
+	# Be careful restoring the old grab, since it's window may not
 	# be visible anymore.
 
 	catch {
-          if {[string equal $Priv(grabStatus) "global"]} {
+	    if {$Priv(grabStatus) eq "global"} {
 		grab set -global $Priv(oldGrab)
 	    } else {
 		grab set $Priv(oldGrab)
@@ -1248,17 +1301,17 @@ proc ::tk::RestoreOldGrab {} {
 
 proc ::tk_menuSetFocus {menu} {
     variable ::tk::Priv
-    if {![info exists Priv(focus)] || [string equal $Priv(focus) {}]} {
+    if {![info exists Priv(focus)] || $Priv(focus) eq ""} {
 	set Priv(focus) [focus]
     }
     focus $menu
 }
-    
+
 proc ::tk::GenerateMenuSelect {menu} {
     variable ::tk::Priv
 
-    if {[string equal $Priv(activeMenu) $menu] \
-          && [string equal $Priv(activeItem) [$menu index active]]} {
+    if {$Priv(activeMenu) eq $menu \
+	    && $Priv(activeItem) eq [$menu index active]} {
 	return
     }
 
@@ -1290,6 +1343,7 @@ proc ::tk_popup {menu x y {entry {}}} {
         tk::SaveGrabInfo $menu
 	grab -global $menu
 	set Priv(popup) $menu
+	set Priv(menuActivated) 1
 	tk_menuSetFocus $menu
     }
 }
