@@ -5,13 +5,11 @@
  *	only event that really does anything is the Quit event.
  *
  * Copyright (c) 1995-1997 Sun Microsystems, Inc.
- * Copyright 2001, Apple Computer, Inc.
- * Copyright (c) 2006 Daniel A. Steffen <das@users.sourceforge.net>
+ * Copyright 2001-2009, Apple Inc.
+ * Copyright (c) 2006-2009 Daniel A. Steffen <das@users.sourceforge.net>
  *
- * See the file "license.terms" for information on usage and redistribution of
- * this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXHLEvents.c,v 1.15.2.1 2008/07/22 17:00:49 das Exp $
+ * See the file "license.terms" for information on usage and redistribution
+ * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  */
 
 #include "tkMacOSXPrivate.h"
@@ -33,22 +31,38 @@ typedef struct KillEvent {
  */
 
 static OSErr		QuitHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static OSErr		OappHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static OSErr		RappHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static OSErr		OdocHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static OSErr		PrintHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static OSErr		ScriptHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static OSErr		PrefsHandler(const AppleEvent *event,
-			    AppleEvent *reply, long handlerRefcon);
+			    AppleEvent *reply, SRefCon handlerRefcon);
 static int		MissedAnyParameters(const AppleEvent *theEvent);
 static int		ReallyKillMe(Tcl_Event *eventPtr, int flags);
 static OSStatus		FSRefToDString(const FSRef *fsref, Tcl_DString *ds);
+
+#pragma mark TKApplication(TKHLEvents)
+
+@implementation TKApplication(TKHLEvents)
+- (void) terminate: (id) sender
+{
+    QuitHandler(NULL, NULL, (SRefCon) _eventInterp);
+}
+
+- (void) preferences: (id) sender
+{
+    PrefsHandler(NULL, NULL, (SRefCon) _eventInterp);
+}
+@end
+
+#pragma mark -
 
 /*
  *----------------------------------------------------------------------
@@ -85,32 +99,32 @@ TkMacOSXInitAppleEvents(
 
 	QuitHandlerUPP = NewAEEventHandlerUPP(QuitHandler);
 	ChkErr(AEInstallEventHandler, kCoreEventClass, kAEQuitApplication,
-		QuitHandlerUPP, (long) interp, false);
+		QuitHandlerUPP, (SRefCon) interp, false);
 
 	OappHandlerUPP = NewAEEventHandlerUPP(OappHandler);
 	ChkErr(AEInstallEventHandler, kCoreEventClass, kAEOpenApplication,
-		OappHandlerUPP, (long) interp, false);
+		OappHandlerUPP, (SRefCon) interp, false);
 
 	RappHandlerUPP = NewAEEventHandlerUPP(RappHandler);
 	ChkErr(AEInstallEventHandler, kCoreEventClass, kAEReopenApplication,
-		RappHandlerUPP, (long) interp, false);
+		RappHandlerUPP, (SRefCon) interp, false);
 
 	OdocHandlerUPP = NewAEEventHandlerUPP(OdocHandler);
 	ChkErr(AEInstallEventHandler, kCoreEventClass, kAEOpenDocuments,
-		OdocHandlerUPP, (long) interp, false);
+		OdocHandlerUPP, (SRefCon) interp, false);
 
 	PrintHandlerUPP = NewAEEventHandlerUPP(PrintHandler);
 	ChkErr(AEInstallEventHandler, kCoreEventClass, kAEPrintDocuments,
-		PrintHandlerUPP, (long) interp, false);
+		PrintHandlerUPP, (SRefCon) interp, false);
 
 	PrefsHandlerUPP = NewAEEventHandlerUPP(PrefsHandler);
 	ChkErr(AEInstallEventHandler, kCoreEventClass, kAEShowPreferences,
-		PrefsHandlerUPP, (long) interp, false);
+		PrefsHandlerUPP, (SRefCon) interp, false);
 
 	if (interp) {
 	    ScriptHandlerUPP = NewAEEventHandlerUPP(ScriptHandler);
 	    ChkErr(AEInstallEventHandler, kAEMiscStandards, kAEDoScript,
-		    ScriptHandlerUPP, (long) interp, false);
+		    ScriptHandlerUPP, (SRefCon) interp, false);
 	}
     }
 }
@@ -133,9 +147,9 @@ TkMacOSXInitAppleEvents(
 
 int
 TkMacOSXDoHLEvent(
-    EventRecord *theEvent)
+    void *theEvent)
 {
-    return AEProcessAppleEvent(theEvent);
+    return AEProcessAppleEvent((EventRecord *)theEvent);
 }
 
 /*
@@ -158,7 +172,7 @@ static OSErr
 QuitHandler(
     const AppleEvent *event,
     AppleEvent *reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
     KillEvent *eventPtr;
@@ -172,7 +186,7 @@ QuitHandler(
 	 * quickly as possible.
 	 */
 
-	eventPtr = (KillEvent *) ckalloc(sizeof(KillEvent));
+	eventPtr = ckalloc(sizeof(KillEvent));
 	eventPtr->header.proc = ReallyKillMe;
 	eventPtr->interp = interp;
 
@@ -201,15 +215,16 @@ static OSErr
 OappHandler(
     const AppleEvent *event,
     AppleEvent *reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     Tcl_CmdInfo dummy;
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
 
     if (interp &&
 	    Tcl_GetCommandInfo(interp, "::tk::mac::OpenApplication", &dummy)){
-	if (Tcl_GlobalEval(interp, "::tk::mac::OpenApplication") != TCL_OK) {
-	    Tcl_BackgroundError(interp);
+	int code = Tcl_EvalEx(interp, "::tk::mac::OpenApplication", -1, TCL_EVAL_GLOBAL);
+	if (code != TCL_OK) {
+	    Tcl_BackgroundException(interp, code);
 	}
     }
     return noErr;
@@ -235,7 +250,7 @@ static OSErr
 RappHandler(
     const AppleEvent *event,
     AppleEvent *reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     Tcl_CmdInfo dummy;
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
@@ -244,8 +259,9 @@ RappHandler(
 
     if (interp && Tcl_GetCommandInfo(interp,
 	    "::tk::mac::ReopenApplication", &dummy)) {
-	if (Tcl_GlobalEval(interp, "::tk::mac::ReopenApplication") != TCL_OK){
-	    Tcl_BackgroundError(interp);
+	int code = Tcl_EvalEx(interp, "::tk::mac::ReopenApplication", -1, TCL_EVAL_GLOBAL);
+	if (code != TCL_OK){
+	    Tcl_BackgroundException(interp, code);
 	}
     }
     return err;
@@ -272,15 +288,16 @@ static OSErr
 PrefsHandler(
     const AppleEvent *event,
     AppleEvent *reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     Tcl_CmdInfo dummy;
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
 
     if (interp &&
 	    Tcl_GetCommandInfo(interp, "::tk::mac::ShowPreferences", &dummy)){
-	if (Tcl_GlobalEval(interp, "::tk::mac::ShowPreferences") != TCL_OK) {
-	    Tcl_BackgroundError(interp);
+	int code = Tcl_EvalEx(interp, "::tk::mac::ShowPreferences", -1, TCL_EVAL_GLOBAL);
+	if (code != TCL_OK) {
+	    Tcl_BackgroundException(interp, code);
 	}
     }
     return noErr;
@@ -306,7 +323,7 @@ static OSErr
 OdocHandler(
     const AppleEvent *event,
     AppleEvent *reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
     AEDescList fileSpecList;
@@ -317,6 +334,7 @@ OdocHandler(
     AEKeyword keyword;
     Tcl_DString command, pathName;
     Tcl_CmdInfo dummy;
+    int code;
 
     /*
      * Don't bother if we don't have an interp or the open document procedure
@@ -329,7 +347,7 @@ OdocHandler(
     }
 
     /*
-     * If we get any errors wil retrieving our parameters we just return with
+     * If we get any errors while retrieving our parameters we just return with
      * no error.
      */
 
@@ -367,9 +385,10 @@ OdocHandler(
      * Now handle the event by evaluating a script.
      */
 
-    if (Tcl_EvalEx(interp, Tcl_DStringValue(&command),
-	    Tcl_DStringLength(&command), TCL_EVAL_GLOBAL) != TCL_OK) {
-	Tcl_BackgroundError(interp);
+    code = Tcl_EvalEx(interp, Tcl_DStringValue(&command),
+	    Tcl_DStringLength(&command), TCL_EVAL_GLOBAL);
+    if (code != TCL_OK) {
+	Tcl_BackgroundException(interp, code);
     }
     Tcl_DStringFree(&command);
     return noErr;
@@ -395,7 +414,7 @@ static OSErr
 PrintHandler(
     const AppleEvent * event,
     AppleEvent * reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
     AEDescList fileSpecList;
@@ -406,6 +425,7 @@ PrintHandler(
     AEKeyword keyword;
     Tcl_DString command, pathName;
     Tcl_CmdInfo dummy;
+    int code;
 
     /*
      * Don't bother if we don't have an interp or the print document procedure
@@ -418,7 +438,7 @@ PrintHandler(
     }
 
     /*
-     * If we get any errors wil retrieving our parameters we just return with
+     * If we get any errors while retrieving our parameters we just return with
      * no error.
      */
 
@@ -447,9 +467,14 @@ PrintHandler(
 	}
     }
 
-    if (Tcl_EvalEx(interp, Tcl_DStringValue(&command),
-	    Tcl_DStringLength(&command), TCL_EVAL_GLOBAL) != TCL_OK) {
-	Tcl_BackgroundError(interp);
+    /*
+     * Now handle the event by evaluating a script.
+     */
+
+    code = Tcl_EvalEx(interp, Tcl_DStringValue(&command),
+	    Tcl_DStringLength(&command), TCL_EVAL_GLOBAL);
+    if (code != TCL_OK) {
+	Tcl_BackgroundException(interp, code);
     }
     Tcl_DStringFree(&command);
     return noErr;
@@ -475,10 +500,11 @@ static OSErr
 ScriptHandler(
     const AppleEvent *event,
     AppleEvent *reply,
-    long handlerRefcon)
+    SRefCon handlerRefcon)
 {
     OSStatus theErr;
     AEDescList theDesc;
+    Size size;
     int tclErr = -1;
     Tcl_Interp *interp = (Tcl_Interp *) handlerRefcon;
     char errString[128];
@@ -491,8 +517,8 @@ ScriptHandler(
     theErr = AEGetParamDesc(event, keyDirectObject, typeWildCard,
 	    &theDesc);
     if (theErr != noErr) {
-	sprintf(errString, "AEDoScriptHandler: GetParamDesc error %ld",
-		theErr);
+	sprintf(errString, "AEDoScriptHandler: GetParamDesc error %d",
+		(int)theErr);
 	theErr = AEPutParamPtr(reply, keyErrorString, typeChar, errString,
 		strlen(errString));
     } else if (MissedAnyParameters(event)) {
@@ -504,66 +530,49 @@ ScriptHandler(
 	AEPutParamPtr(reply, keyErrorString, typeChar, errString,
 		strlen(errString));
 	theErr = -1771;
-    } else if (theDesc.descriptorType == (DescType) typeChar) {
-	/*
-	 * We've had some data sent to us. Evaluate it.
-	 */
-
-	Tcl_DString encodedText;
-	short i;
-	Size size = AEGetDescDataSize(&theDesc);
-	char *data = ckalloc(size + 1);
-
-	AEGetDescData(&theDesc, data, size);
-	data[size] = 0;
-	for (i = 0; i < size; i++) {
-	    if (data[i] == '\r') {
-		data[i] = '\n';
-	    }
-	}
-	AEReplaceDescData(theDesc.descriptorType, data, size + 1, &theDesc);
-	Tcl_ExternalToUtfDString(NULL, data, size, &encodedText);
-	tclErr = Tcl_EvalEx(interp, Tcl_DStringValue(&encodedText),
-		Tcl_DStringLength(&encodedText), TCL_EVAL_GLOBAL);
-	Tcl_DStringFree(&encodedText);
-    } else if (theDesc.descriptorType == (DescType) typeAlias) {
+    } else if (theDesc.descriptorType == (DescType) typeAlias &&
+	    AEGetParamPtr(event, keyDirectObject, typeFSRef, NULL, NULL,
+	    0, &size) == noErr && size == sizeof(FSRef)) {
 	/*
 	 * We've had a file sent to us. Source it.
 	 */
 
-	Boolean dummy;
 	FSRef file;
-	Size theSize = AEGetDescDataSize(&theDesc);
-	AliasPtr alias = (AliasPtr) ckalloc(theSize);
-
-	if (alias) {
-	    AEGetDescData(&theDesc, alias, theSize);
-
-	    theErr = FSResolveAlias(NULL, &alias, &file, &dummy);
-	    ckfree((char*)alias);
-	} else {
-	    theErr = memFullErr;
-	}
+	theErr = AEGetParamPtr(event, keyDirectObject, typeFSRef, NULL, &file,
+		size, NULL);
 	if (theErr == noErr) {
 	    Tcl_DString scriptName;
 
 	    theErr = FSRefToDString(&file, &scriptName);
 	    if (theErr == noErr) {
-		Tcl_EvalFile(interp, Tcl_DStringValue(&scriptName));
+		tclErr = Tcl_FSEvalFileEx(interp, Tcl_DStringValue(&scriptName), NULL);
 		Tcl_DStringFree(&scriptName);
+	    } else {
+		sprintf(errString, "AEDoScriptHandler: file not found");
+		AEPutParamPtr(reply, keyErrorString, typeChar, errString,
+			strlen(errString));
 	    }
-	} else {
-	    sprintf(errString, "AEDoScriptHandler: file not found");
-	    AEPutParamPtr(reply, keyErrorString, typeChar, errString,
-		    strlen(errString));
+	}
+    } else if (AEGetParamPtr(event, keyDirectObject, typeUTF8Text, NULL, NULL,
+	    0, &size) == noErr && size) {
+	/*
+	 * We've had some data sent to us. Evaluate it.
+	 */
+
+	char *data = ckalloc(size + 1);
+	theErr = AEGetParamPtr(event, keyDirectObject, typeUTF8Text, NULL, data,
+		size, NULL);
+	if (theErr == noErr) {
+	    tclErr = Tcl_EvalEx(interp, data, size, TCL_EVAL_GLOBAL);
 	}
     } else {
 	/*
 	 * Umm, don't recognize what we've got...
 	 */
 
-	sprintf(errString, "AEDoScriptHandler: invalid script type '%-4.4s',"
-		" must be 'alis' or 'TEXT'", (char*) &theDesc.descriptorType);
+	sprintf(errString, "AEDoScriptHandler: invalid script type '%-4.4s', "
+		"must be 'alis' or coercable to 'utf8'",
+		(char*) &theDesc.descriptorType);
 	AEPutParamPtr(reply, keyErrorString, typeChar, errString,
 		strlen(errString));
 	theErr = -1770;
@@ -582,7 +591,7 @@ ScriptHandler(
 	    AEPutParamPtr(reply, keyDirectObject, typeChar, result, reslen);
 	} else {
 	    AEPutParamPtr(reply, keyErrorString, typeChar, result, reslen);
-	    AEPutParamPtr(reply, keyErrorNumber, typeInteger, (Ptr) &tclErr,
+	    AEPutParamPtr(reply, keyErrorNumber, typeSInt32, (Ptr) &tclErr,
 		    sizeof(int));
 	}
     }
@@ -616,13 +625,14 @@ ReallyKillMe(
     Tcl_Interp *interp = ((KillEvent *) eventPtr)->interp;
     Tcl_CmdInfo dummy;
     int quit = Tcl_GetCommandInfo(interp, "::tk::mac::Quit", &dummy);
+    int code = Tcl_EvalEx(interp, quit ? "::tk::mac::Quit" : "exit", -1, TCL_EVAL_GLOBAL);
 
-    if (Tcl_GlobalEval(interp, quit ? "::tk::mac::Quit" : "exit") != TCL_OK) {
+    if (code != TCL_OK) {
 	/*
 	 * Should be never reached...
 	 */
 
-	Tcl_BackgroundError(interp);
+	Tcl_BackgroundException(interp, code);
     }
     return 1;
 }
@@ -651,7 +661,7 @@ MissedAnyParameters(
    Size actualSize;
    OSStatus err;
 
-   err = ChkErr(AEGetAttributePtr, theEvent, keyMissedKeywordAttr,
+   err = AEGetAttributePtr(theEvent, keyMissedKeywordAttr,
 	    typeWildCard, &returnedType, NULL, 0, &actualSize);
 
    return (err != errAEDescNotFound);
@@ -690,8 +700,9 @@ FSRefToDString(
 
 /*
  * Local Variables:
- * mode: c
+ * mode: objc
  * c-basic-offset: 4
- * fill-column: 78
+ * fill-column: 79
+ * coding: utf-8
  * End:
  */
