@@ -1,37 +1,21 @@
 /*
  * tkMacOSXPort.h --
  *
- *	This file is included by all of the Tk C files.  It contains
+ *	This file is included by all of the Tk C files. It contains
  *	information that may be configuration-dependent, such as
  *	#includes for system include files and a few other things.
  *
  * Copyright (c) 1994-1996 Sun Microsystems, Inc.
- * Copyright 2001, Apple Computer, Inc.
+ * Copyright 2001-2009, Apple Inc.
+ * Copyright (c) 2005-2009 Daniel A. Steffen <das@users.sourceforge.net>
  *
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
- *
- * RCS: @(#) $Id: tkMacOSXPort.h,v 1.3.2.6 2006/09/10 17:07:36 das Exp $
  */
 
 #ifndef _TKMACPORT
 #define _TKMACPORT
 
-/*
- * Macro to use instead of "void" for arguments that must have
- * type "void *" in ANSI C;  maps them to type "char *" in
- * non-ANSI systems.  This macro may be used in some of the include
- * files below, which is why it is defined here.
- */
-
-#ifndef VOID
-#   ifdef __STDC__
-#       define VOID void
-#   else
-#       define VOID char
-#   endif
-#endif
-
 #include <stdio.h>
 #include <ctype.h>
 #include <fcntl.h>
@@ -54,11 +38,14 @@
 #   include <time.h>
 #else
 #   if HAVE_SYS_TIME_H
-#       include <sys/time.h>
+#	include <sys/time.h>
 #   else
-#       include <time.h>
+#	include <time.h>
 #   endif
 #endif
+#if HAVE_INTTYPES_H
+#    include <inttypes.h>
+#endif
 #include <unistd.h>
 #include <X11/Xlib.h>
 #include <X11/cursorfont.h>
@@ -113,20 +100,6 @@
 #   define NBBY 8
 #endif
 
-/*
- * Declarations for various library procedures that may not be declared
- * in any other header file.
- */
-
-#ifndef strcasecmp
-extern int		strcasecmp _ANSI_ARGS_((CONST char *s1,
-			    CONST char *s2));
-#endif
-#ifndef strncasecmp			    
-extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
-			    CONST char *s2, size_t n));
-#endif
-
 /*
  * The following define causes Tk to use its internal keysym hash table
  */
@@ -139,7 +112,7 @@ extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
  */
 
 #define XFlush(display)
-#define XFree(data) {if ((data) != NULL) ckfree((char *) (data));}
+#define XFree(data) {if ((data) != NULL) ckfree(data);}
 #define XGrabServer(display)
 #define XNoOp(display) {display->request++;}
 #define XUngrabServer(display)
@@ -150,9 +123,6 @@ extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
  * The following functions are not used on the Mac, so we stub them out.
  */
 
-#define TkFreeWindowId(dispPtr,w)
-#define TkInitXId(dispPtr)
-#define TkpButtonSetDefaults(specPtr) {}
 #define TkpCmapStressed(tkwin,colormap) (0)
 #define TkpFreeColor(tkColPtr)
 #define TkSetPixmapColormap(p,c) {}
@@ -176,7 +146,7 @@ extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
 
 #define TkpPrintWindowId(buf,w) \
 	sprintf((buf), "0x%x", (unsigned int) (w))
-	    
+
 /*
  * TkpScanWindowId is just an alias for Tcl_GetInt on Unix.
  */
@@ -185,9 +155,19 @@ extern int		strncasecmp _ANSI_ARGS_((CONST char *s1,
 	Tcl_GetInt((i),(s),(int *) (wp))
 
 /*
- * Magic pixel values for dynamic (or active) colors.
+ * Turn off Tk double-buffering as Aqua windows are already double-buffered.
  */
 
+#define TK_NO_DOUBLE_BUFFERING 1
+
+/*
+ * Magic pixel code values for system colors.
+ *
+ * NOTE: values must be kept in sync with indices into the
+ *	 systemColorMap array in tkMacOSXColor.c !
+ */
+
+#define TRANSPARENT_PIXEL		30
 #define HIGHLIGHT_PIXEL			31
 #define HIGHLIGHT_SECONDARY_PIXEL	32
 #define HIGHLIGHT_TEXT_PIXEL		33
