@@ -294,14 +294,14 @@ static Ttk_StateTable TabPositionTable[] = {
  *       TP30000359-TPXREF116>
  */
 
+static const int TAB_HEIGHT = 10;
+static const int TAB_OVERLAP = 10;
 
 static void TabElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
-     *heightPtr = GetThemeMetric(kThemeMetricLargeTabHeight, heightPtr);
-     *paddingPtr = Ttk_MakePadding(0, 0, 0, 2);
-
+    *heightPtr = TAB_HEIGHT + TAB_OVERLAP - 1;
 }
 
 static void TabElementDraw(
@@ -319,6 +319,7 @@ static void TabElementDraw(
 	.position = Ttk_StateTableLookup(TabPositionTable, state),
     };
 
+    bounds.size.height += TAB_OVERLAP;
     BEGIN_DRAWING(d)
     ChkErr(HIThemeDrawTab, &bounds, &info, dc.context, HIOrientation, NULL);
     END_DRAWING
@@ -356,8 +357,8 @@ static void PaneElementDraw(
 	.adornment = kHIThemeTabPaneAdornmentNormal,
     };
 
-    bounds.origin.y -= kThemeMetricTabFrameOverlap;
-    bounds.size.height += kThemeMetricTabFrameOverlap;
+    bounds.origin.y -= TAB_OVERLAP;
+    bounds.size.height += TAB_OVERLAP;
     BEGIN_DRAWING(d)
     ChkErr(HIThemeDrawTabPane, &bounds, &info, dc.context, HIOrientation);
     END_DRAWING
